@@ -1,6 +1,6 @@
 /* Routines for name->symbol lookups in GDB.
 
-   Copyright (C) 2003-2025 Free Software Foundation, Inc.
+   Copyright (C) 2003-2026 Free Software Foundation, Inc.
 
    Contributed by David Carlton <carlton@bactrian.org> and by Kealia,
    Inc.
@@ -911,24 +911,20 @@ struct multidictionary
   /* The number of language dictionaries currently allocated.
      Only used for expandable dictionaries.  */
   unsigned short n_allocated_dictionaries;
+
+  /* The type of dictionary.  */
+  enum dict_type type;
 };
 
 /* A helper function to collate symbols on the pending list by language.  */
 
 static gdb::unordered_map<enum language, std::vector<symbol *>>
-collate_pending_symbols_by_language (const struct pending *symbol_list)
+collate_pending_symbols_by_language (const std::vector<symbol *> &symbol_list)
 {
   gdb::unordered_map<enum language, std::vector<symbol *>> nsyms;
 
-  for (const pending *list_counter = symbol_list;
-       list_counter != nullptr; list_counter = list_counter->next)
-    {
-      for (int i = list_counter->nsyms - 1; i >= 0; --i)
-	{
-	  enum language language = list_counter->symbol[i]->language ();
-	  nsyms[language].push_back (list_counter->symbol[i]);
-	}
-    }
+  for (auto it = symbol_list.rbegin (); it != symbol_list.rend (); ++it)
+    nsyms[(*it)->language ()].push_back (*it);
 
   return nsyms;
 }
@@ -937,7 +933,7 @@ collate_pending_symbols_by_language (const struct pending *symbol_list)
 
 struct multidictionary *
 mdict_create_hashed (struct obstack *obstack,
-		     const struct pending *symbol_list)
+		     const std::vector<symbol *> &symbol_list)
 {
   struct multidictionary *retval
     = XOBNEW (obstack, struct multidictionary);
@@ -948,6 +944,7 @@ mdict_create_hashed (struct obstack *obstack,
   retval->dictionaries
     = XOBNEWVEC (obstack, struct dictionary *, nsyms.size ());
   retval->n_allocated_dictionaries = nsyms.size ();
+  retval->type = DICT_HASHED;
 
   int idx = 0;
   for (const auto &[language, symlist] : nsyms)
@@ -969,6 +966,7 @@ mdict_create_hashed_expandable (enum language language)
   retval->n_allocated_dictionaries = 1;
   retval->dictionaries = XNEW (struct dictionary *);
   retval->dictionaries[0] = dict_create_hashed_expandable (language);
+  retval->type = DICT_HASHED_EXPANDABLE;
 
   return retval;
 }
@@ -977,7 +975,7 @@ mdict_create_hashed_expandable (enum language language)
 
 struct multidictionary *
 mdict_create_linear (struct obstack *obstack,
-		     const struct pending *symbol_list)
+		     const std::vector<symbol *> &symbol_list)
 {
   struct multidictionary *retval
     = XOBNEW (obstack, struct multidictionary);
@@ -988,6 +986,7 @@ mdict_create_linear (struct obstack *obstack,
   retval->dictionaries
     = XOBNEWVEC (obstack, struct dictionary *, nsyms.size ());
   retval->n_allocated_dictionaries = nsyms.size ();
+  retval->type = DICT_LINEAR;
 
   int idx = 0;
   for (const auto &[language, symlist] : nsyms)
@@ -1009,6 +1008,7 @@ mdict_create_linear_expandable (enum language language)
   retval->n_allocated_dictionaries = 1;
   retval->dictionaries = XNEW (struct dictionary *);
   retval->dictionaries[0] = dict_create_linear_expandable (language);
+  retval->type = DICT_LINEAR_EXPANDABLE;
 
   return retval;
 }
@@ -1018,15 +1018,12 @@ mdict_create_linear_expandable (enum language language)
 void
 mdict_free (struct multidictionary *mdict)
 {
-  /* Grab the type of dictionary being used.  */
-  enum dict_type type = mdict->dictionaries[0]->vector->type;
-
   /* Loop over all dictionaries and free them.  */
   for (unsigned short idx = 0; idx < mdict->n_allocated_dictionaries; ++idx)
     dict_free (mdict->dictionaries[idx]);
 
   /* Free the dictionary list, if needed.  */
-  switch (type)
+  switch (mdict->type)
     {
     case DICT_HASHED:
     case DICT_LINEAR:
@@ -1036,6 +1033,7 @@ mdict_free (struct multidictionary *mdict)
     case DICT_HASHED_EXPANDABLE:
     case DICT_LINEAR_EXPANDABLE:
       xfree (mdict->dictionaries);
+      xfree (mdict);
       break;
     }
 }
@@ -1067,10 +1065,7 @@ create_new_language_dictionary (struct multidictionary *mdict,
 {
   struct dictionary *retval = nullptr;
 
-  /* We use the first dictionary entry to decide what create function
-     to call.  Not optimal but sufficient.  */
-  gdb_assert (mdict->dictionaries[0] != nullptr);
-  switch (mdict->dictionaries[0]->vector->type)
+  switch (mdict->type)
     {
     case DICT_HASHED:
     case DICT_LINEAR:
@@ -1118,7 +1113,7 @@ mdict_add_symbol (struct multidictionary *mdict, struct symbol *sym)
 
 void
 mdict_add_pending (struct multidictionary *mdict,
-		   const struct pending *symbol_list)
+		   const std::vector<symbol *> &symbol_list)
 {
   gdb::unordered_map<enum language, std::vector<symbol *>> nsyms
     = collate_pending_symbols_by_language (symbol_list);
