@@ -1,6 +1,6 @@
 /* DTrace probe support for GDB.
 
-   Copyright (C) 2014-2022 Free Software Foundation, Inc.
+   Copyright (C) 2014-2026 Free Software Foundation, Inc.
 
    Contributed by Oracle, Inc.
 
@@ -19,7 +19,7 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
+#include "extract-store-integer.h"
 #include "probe.h"
 #include "elf-bfd.h"
 #include "gdbtypes.h"
@@ -129,7 +129,7 @@ public:
 
   /* See probe.h.  */
   struct value *evaluate_argument (unsigned n,
-				   frame_info_ptr frame) override;
+				   const frame_info_ptr &frame) override;
 
   /* See probe.h.  */
   void compile_to_ax (struct agent_expr *aexpr,
@@ -241,21 +241,21 @@ struct dtrace_dof_hdr
   /* Identification bytes (see above). */
   uint8_t dofh_ident[16];
   /* File attribute flags (if any). */
-  uint32_t dofh_flags;   
+  uint32_t dofh_flags;
   /* Size of file header in bytes. */
-  uint32_t dofh_hdrsize; 
+  uint32_t dofh_hdrsize;
   /* Size of section header in bytes. */
-  uint32_t dofh_secsize; 
+  uint32_t dofh_secsize;
   /* Number of section headers. */
-  uint32_t dofh_secnum;  
+  uint32_t dofh_secnum;
   /* File offset of section headers. */
-  uint64_t dofh_secoff;  
+  uint64_t dofh_secoff;
   /* File size of loadable portion. */
-  uint64_t dofh_loadsz;  
+  uint64_t dofh_loadsz;
   /* File size of entire DOF file. */
-  uint64_t dofh_filesz;  
+  uint64_t dofh_filesz;
   /* Reserved for future use. */
-  uint64_t dofh_pad;     
+  uint64_t dofh_pad;
 };
 
 /* A DOF section, whose contents depend on its type.  The several
@@ -267,15 +267,15 @@ struct dtrace_dof_sect
   /* Section type (see the define above). */
   uint32_t dofs_type;
   /* Section data memory alignment. */
-  uint32_t dofs_align; 
+  uint32_t dofs_align;
   /* Section flags (if any). */
-  uint32_t dofs_flags; 
+  uint32_t dofs_flags;
   /* Size of section entry (if table). */
   uint32_t dofs_entsize;
   /* DOF + offset points to the section data. */
   uint64_t dofs_offset;
   /* Size of section data in bytes.  */
-  uint64_t dofs_size;  
+  uint64_t dofs_size;
 };
 
 /* A DOF provider, which is the provider of a probe.  */
@@ -283,19 +283,19 @@ struct dtrace_dof_sect
 struct dtrace_dof_provider
 {
   /* Link to a DTRACE_DOF_SECT_TYPE_STRTAB section. */
-  uint32_t dofpv_strtab; 
+  uint32_t dofpv_strtab;
   /* Link to a DTRACE_DOF_SECT_TYPE_PROBES section. */
-  uint32_t dofpv_probes; 
+  uint32_t dofpv_probes;
   /* Link to a DTRACE_DOF_SECT_TYPE_PRARGS section. */
-  uint32_t dofpv_prargs; 
+  uint32_t dofpv_prargs;
   /* Link to a DTRACE_DOF_SECT_TYPE_PROFFS section. */
-  uint32_t dofpv_proffs; 
+  uint32_t dofpv_proffs;
   /* Provider name string. */
-  uint32_t dofpv_name;   
+  uint32_t dofpv_name;
   /* Provider attributes. */
   uint32_t dofpv_provattr;
   /* Module attributes. */
-  uint32_t dofpv_modattr; 
+  uint32_t dofpv_modattr;
   /* Function attributes. */
   uint32_t dofpv_funcattr;
   /* Name attributes. */
@@ -315,33 +315,33 @@ struct dtrace_dof_provider
 struct dtrace_dof_probe
 {
   /* Probe base address or offset. */
-  uint64_t dofpr_addr;   
+  uint64_t dofpr_addr;
   /* Probe function string. */
-  uint32_t dofpr_func;   
+  uint32_t dofpr_func;
   /* Probe name string. */
-  uint32_t dofpr_name;   
+  uint32_t dofpr_name;
   /* Native argument type strings. */
-  uint32_t dofpr_nargv;  
+  uint32_t dofpr_nargv;
   /* Translated argument type strings. */
-  uint32_t dofpr_xargv;  
+  uint32_t dofpr_xargv;
   /* Index of first argument mapping. */
-  uint32_t dofpr_argidx; 
+  uint32_t dofpr_argidx;
   /* Index of first offset entry. */
-  uint32_t dofpr_offidx; 
+  uint32_t dofpr_offidx;
   /* Native argument count. */
-  uint8_t  dofpr_nargc;  
+  uint8_t  dofpr_nargc;
   /* Translated argument count. */
-  uint8_t  dofpr_xargc;  
+  uint8_t  dofpr_xargc;
   /* Number of offset entries for probe. */
-  uint16_t dofpr_noffs;  
+  uint16_t dofpr_noffs;
   /* Index of first is-enabled offset. */
   uint32_t dofpr_enoffidx;
   /* Number of is-enabled offsets. */
   uint16_t dofpr_nenoffs;
   /* Reserved for future use. */
-  uint16_t dofpr_pad1;   
+  uint16_t dofpr_pad1;
   /* Reserved for future use. */
-  uint32_t dofpr_pad2;   
+  uint32_t dofpr_pad2;
 };
 
 /* DOF supports two different encodings: MSB (big-endian) and LSB
@@ -424,7 +424,7 @@ dtrace_process_dof_probe (struct objfile *objfile,
 
      It follows that if there are DTrace is-enabled probes defined for
      some provider/name but no DTrace regular probes defined then the
-     GDB user wont be able to enable/disable these conditionals.  */
+     GDB user won't be able to enable/disable these conditionals.  */
 
   num_probes = DOF_UINT (dof, probe->dofpr_noffs);
   if (num_probes == 0)
@@ -493,8 +493,8 @@ dtrace_process_dof_probe (struct objfile *objfile,
 	    {
 	    }
 
-	  if (expr != NULL && expr->first_opcode () == OP_TYPE)
-	    type = value_type (evaluate_type (expr.get ()));
+	  if (expr != NULL && expr->type_p ())
+	    type = expr->evaluate_type ()->type ();
 
 	  args.emplace_back (type, std::move (type_str), std::move (expr));
 	}
@@ -605,7 +605,7 @@ dtrace_process_dof (asection *sect, struct objfile *objfile,
       }
 
   return;
-	  
+
  invalid_dof_data:
   complaint (_("skipping section '%s' which does not contain valid DOF data."),
 	     sect->name);
@@ -708,13 +708,13 @@ dtrace_probe::can_evaluate_arguments () const
 
 struct value *
 dtrace_probe::evaluate_argument (unsigned n,
-				 frame_info_ptr frame)
+				 const frame_info_ptr &frame)
 {
   struct gdbarch *gdbarch = this->get_gdbarch ();
   struct dtrace_probe_arg *arg;
 
   arg = this->get_arg_by_number (n, gdbarch);
-  return evaluate_expression (arg->expr.get (), arg->type);
+  return arg->expr->evaluate (arg->type);
 }
 
 /* Implementation of the compile_to_ax method.  */
@@ -889,9 +889,7 @@ info_probes_dtrace_command (const char *arg, int from_tty)
   info_probes_for_spops (arg, from_tty, &dtrace_static_probe_ops);
 }
 
-void _initialize_dtrace_probe ();
-void
-_initialize_dtrace_probe ()
+INIT_GDB_FILE (dtrace_probe)
 {
   all_static_probe_ops.push_back (&dtrace_static_probe_ops);
 
