@@ -1,5 +1,5 @@
 /* GDB self-test for each gdbarch.
-   Copyright (C) 2017-2022 Free Software Foundation, Inc.
+   Copyright (C) 2017-2026 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -16,8 +16,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#ifndef SELFTEST_ARCH_H
-#define SELFTEST_ARCH_H
+#ifndef GDB_SELFTEST_ARCH_H
+#define GDB_SELFTEST_ARCH_H
 
 typedef void self_test_foreach_arch_function (struct gdbarch *);
 
@@ -29,6 +29,11 @@ namespace selftests
 extern void
   register_test_foreach_arch (const std::string &name,
 			      self_test_foreach_arch_function *function);
+
+/* Return true if GDBARCH should be skipped in some selftests to avoid
+   warnings.  */
+
+extern bool selftest_skip_warning_arch (struct gdbarch *gdbarch);
 }
 
-#endif /* SELFTEST_ARCH_H */
+#endif /* GDB_SELFTEST_ARCH_H */
