@@ -1,6 +1,6 @@
 /* This testcase is part of GDB, the GNU debugger.
 
-   Copyright 2017-2019 Free Software Foundation, Inc.
+   Copyright 2017-2026 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 
 static void __attribute__ ((used)) ambiguous_fun (void) {}
 
-static int ambiguous_var;
+static int __attribute__ ((used)) ambiguous_var;
 
 
 
@@ -37,6 +37,6 @@ static int ambiguous_var;
 
 int
 main (void)
-{
+{ /* main prologue */
   return 0;
 }
